#!/usr/bin/perl
#
#    Cluster Systems Management Webmin Module
#    Copyright (C) 2002 by IBM Corp.
#
# Display and modify sensors

use strict;
BEGIN { $::CSMDIR = '..'; }
use lib "$::CSMDIR";
require GuiUtils;
GuiUtils->init();

$ENV{'CT_MANAGEMENT_SCOPE'}=1;

my $isAction = exists($::in{dynTestButton});
GuiUtils->addHeader(undef, 3, 'Sensors');

#todo: also support a button to force the running of a sensor to test it.  This is only necessary
#      in catskill, since in beacon and before lssensor caused it to be run.  In catskill, you need
#      to set ControlFlags to 1 before running lssensor to force the sensor to be executed.  (refsensor
#      will only run the sensor if it is being monitored at the time.)

# Process the buttons
#if (exists($::in{trySelectButton}))
#  {
#	print "<H3>Select String Query:  $::in{selectString}</H3>\n";
#	GuiUtils->runcmd("$GuiUtils::csmroot/bin/$lsCmd -w " . NodeUtils->quote($::in{selectString}), 1);
#	print "<P></P>\n";
#	&footer("nodegrp.cgi?groupName=$::in{groupName}&groupType=$::in{groupType}&selectString=$::in{selectString}&device=$::in{device}", "CSM $memName Groups");
#	return;
#  }

my $sensor;
my $changedAttrStr;
my %input;
my %values;

# Get list of sensors
my $sensorHash = getSensorHash();

if (exists($::in{chsensorButton}))           # creating a sensor or changing properties
  {
	my %orig = split(/:\|:/, $::in{origValues});      # these are the attribute values the sensor had before the user typed in new ones

	# get the value of all the input fields
	foreach my $k (keys %::in) { my $a; if (($a) = $k=~/^input(.+)$/) { $input{$a} = NodeUtils->quote($::in{$k}); } }
	$sensor = $input{Name};

	# figure out which values were changed by the user
	my %changedAttrs;
	foreach my $k (keys %input) { if ($input{$k} ne $orig{$k}) { $changedAttrs{$k} = $input{$k}; } }

	
	if ($changedAttrs{Name}) {       # if they changed the name, we are creating a new sensor
	  my $rc = createSensor($sensor);
	  if (!$rc) { $changedAttrStr = "Sensor $sensor created."; }
	  else { %values = %input; }          # copy the input values back into values so we preserve the users changes
	}
	
	else {     # change properties
	  my @changedKeys = keys %changedAttrs;
	  if (scalar(@changedKeys)==1 && $changedKeys[0] eq 'RefreshInterval' && $$sensorHash{$sensor}) {     # this is the only attribute chsensor can currently change
		my $outref = GuiUtils->runcmd("/usr/bin/chsensor -i $input{RefreshInterval} $sensor");
		if (!$::RUNCMD_RC) { $changedAttrStr = 'Properties changed: ' . join(', ', @changedKeys); }
		else { %values = %input; }          # copy the input values back into values so we preserve the users changes
	  }
	  elsif (!$$sensorHash{$sensor}) {       # the sensor is gone already so just create it
		my $rc = createSensor($sensor);
		if (!$rc) { $changedAttrStr = "Sensor $sensor created."; }
		else { %values = %input; }          # copy the input values back into values so we preserve the users changes
	  }
	  else {       # to change the other attrs we have to delete and recreate the sensor
		GuiUtils->msg('I', "To change some of the properties, $sensor must be removed and recreated.");
		GuiUtils->runcmd("/usr/bin/rmsensor $sensor");
		if ($::RUNCMD_RC == 0 || $::RUNCMD_RC == 6) {
		  delete $$sensorHash{$sensor};
		  my $rc = createSensor($sensor);
		  if (!$rc) { $changedAttrStr = "$sensor succesfully recreated. Properties changed: " . join(', ', @changedKeys); }
		  else { %values = %input; }          # copy the input values back into values so we preserve the users changes
		}
	  }
	}
  }
elsif (exists($::in{delsensorButton}))
  {
	$sensor = NodeUtils->quote($::in{inputName});
	GuiUtils->runcmd("/usr/bin/rmsensor $sensor");
	if ($::RUNCMD_RC == 0) { delete $$sensorHash{$sensor};  $changedAttrStr = "$sensor succesfully removed.";  $sensor=''; }
	#else $sensor has the proper value
  }
else     # only displaying
  {
	$sensor = NodeUtils->quote($::in{sensor});
  }

my @sensorList = sort keys %$sensorHash;
foreach my $s (@sensorList) { $s = GuiUtils->unquote($s); }

# Get the attributes for the selected node group
if (!empty($sensor) && !scalar(keys(%values)))
  {
	#my $outref = GuiUtils->runcmd("/usr/bin/lssensor $::in{sensor}");
	my $outref = GuiUtils->runcmd(qq(/usr/bin/lsrsrc-api -D ':|:' -in -s IBM.Sensor::) . NodeUtils->quote("Name==$sensor") . qq(::'*b0x0020'));
	%values = split(/:\|:/, $$outref[0]);      # this puts the pairs of attr name and value in the hash correctly
	foreach my $k (keys %values) { $values{$k} = NodeUtils->quote($values{$k}); }           # quote the values so they do not mess up the html
  }
else            # no sensor selected
  {
	# handle the case where we are invoked from the action button on the main page
  }
	
my $code = GuiUtils->processHTML('sensor-src.html');
eval $code;
if (length($@)) { print "<P>$@</P>"; }

&footer('.', 'Monitoring');


sub createSensor {
  my $s = shift @_;
  #todo:  use mksensor when it supports setting UserName and Description
  my $cmd = "/usr/bin/mkrsrc-api IBM.Sensor::Name::$s";

  # apply the defaults that mksensor does
  my $username;
  if (!empty($input{UserName})) { $username = $input{UserName}; }
  else { $username = `whoami`;    chop $username; }
  my $interval = (!empty($input{RefreshInterval})) ? $input{RefreshInterval} : 60;
  $cmd .= "::Command::$input{Command}::UserName::${username}::RefreshInterval::${interval}";

  if (!empty($input{ErrorExitValue})) { $cmd .= "::ErrorExitValue::$input{ErrorExitValue}"; }
  if (!empty($input{Description})) { $cmd .= "::Description::$input{Description}"; }
  my $outref = GuiUtils->runcmd($cmd);
  if (!$::RUNCMD_RC) { $$sensorHash{$s} = 1; }     # update our lists of sensors
  return $::RUNCMD_RC;
}


sub empty {
  my $s = GuiUtils->unquote(shift @_);
  return (!length($s));
}


sub getSensorHash {
  my $sensorList = GuiUtils->runcmd("/usr/bin/lssensor");
  my $sensorHash = {};
  foreach my $s (@$sensorList) { $$sensorHash{NodeUtils->quote($s)} = 1; }
  return $sensorHash;
}


# not used...
sub sensorExists {
  my $sensor = shift @_;
  GuiUtils->runcmd('/usr/bin/lsrsrc-api -s IBM.Sensor::' . NodeUtils->quote("Name==$sensor") . '::Name', 2);
  return !$::RUNCMD_RC;
}
