#!/usr/bin/perl
#
#    Cluster Systems Management Webmin Module
#    Copyright (C) 2002 by IBM Corp.
#
# Get input for definenode
# This file is also used to define devices

use strict;
BEGIN { $::DEV = $0=~m|csm/device|; $::CSMDIR = $::DEV ? '..' : '.'; }
use lib "$::CSMDIR";
require GuiUtils;
GuiUtils->init();
my $defCmd = $::DEV ? 'definehwdev' : 'definenode';
my $nodeOpt = $::DEV ? '-d' : '-n';

GuiUtils->addHeader(undef, $::DEV, $::DEV?'Define Devices':'Define Nodes');

my %values;
my $modifyNodesChecked = 'false';

# If they pushed the Define Node button, run definenode
if (exists($::in{defineFromValButton}) || exists($::in{createNodedefButton}))       # they pushed the Define from Values or Create Nodedef button
  {
	my $isCreate = exists($::in{createNodedefButton});
	if ($isCreate) { delete $::in{createNodedefButton}; }
	else { delete $::in{defineFromValButton}; }

	if ($::DEV && !length($::in{StatusMethod})) { $::in{StatusMethod} = 'ping'; }

	# Copy the attr values into the values hash so they will be filled into
	# the text fields
	%values = %::in;
	if (exists($::in{modifyNodesCheckbox})) { $modifyNodesChecked = 'true'; delete($::in{modifyNodesCheckbox}); }
	my $cmd = "$GuiUtils::csmroot/bin/$defCmd";
	$cmd .= " $nodeOpt $::in{Hostname}";
	delete $::in{Hostname};
	if (exists($::in{Count}))
	  {
		if ($::in{Count} =~ /\S/) { $cmd .= " -c $::in{Count}"; }
		delete $::in{Count};
	  }
	if ($::in{HWControlPoint}=~/[:,]/)
	  {
		$cmd .= ' -H ' . NodeUtils->quote($::in{HWControlPoint});
		delete $::in{HWControlPoint};
	  }
	if ($::in{ConsoleServerName}=~/[:,]/)
	  {
		$cmd .= ' -C ' . NodeUtils->quote($::in{ConsoleServerName});
		delete $::in{ConsoleServerName};
	  }

	if ($isCreate) { $cmd .= ' -s'; }
	
	foreach my $key (keys %::in){ #the leftovers are all attributes
	  if($::in{$key} =~ m/\S/ && $key ne 'nodedefField'){ #if it matches a non-whitespace character
		$cmd .= " $key=" . NodeUtils->quote($::in{$key});
	  }
	}

	if ($isCreate) { $cmd .= ' > ' . NodeUtils->quote($::in{nodedefField}); }
	
	GuiUtils->runcmd($cmd, 1);
	if ($::RUNCMD_RC)          # the error msg is in the nodedef file
	  {
		my @output = NodeUtils->readFile($::in{nodedefField});
		foreach (@output) { print "$_<BR>\n"; }
	  }
  }
elsif (exists($::in{nodedefButton}))       # they pushed the Define from Nodedef button
  {
	$values{nodedefField} = $::in{nodedefField};
	my $cmd = "$GuiUtils::csmroot/bin/$defCmd";
	if (exists($::in{modifyNodesCheckbox})) {
	  $cmd .= ' -m';
	  $modifyNodesChecked = 'true';
	}
	$cmd .= ' -f ' . NodeUtils->quote($::in{nodedefField});
	GuiUtils->runcmd($cmd, 1);
  }
elsif (length($::in{nodes}))       # prime the textfields with values from this node
  {
	#my @node_array = split(/,/, $::in{nodes});
	my $nodearray = GuiUtils->processNodelist($::in{nodes}, $::DEV?(Device=>1,listNodeAttrs=>1):(listNodeAttrs=>1));
	if (defined($nodearray) && scalar(@$nodearray)) {
	  $#$nodearray = 0;   # delete all but the 1st element
	  #print "<p>#=", scalar(@$nodearray), " 1st=$$nodearray[0]</p>\n";
	  my %opthash = (IncludeAttrNames => 1);
	  if ($::DEV) { $opthash{ResourceClass} = 'IBM.DeviceHwCtrl'; }
	  $::VERBOSE = $::GUI_VERBOSE;          # this will cause NodeUtils to show the cmds
	  my $att_ref = NodeUtils->listNodeAttrs($nodearray, '', \%opthash);
	  %values = split(/:\|:/, $$att_ref[0]);    # take the alternating attr names/values and put them in the hash
	}
  }
else
  {
	#todo: this only works if the mgmt svr is not also defined as a node
	# Get default values for definenode from the mgmt svr
	#my $hostname = (`hostname`)[0];
	#my $output = GuiUtils->runcmd("$GuiUtils::csmroot/bin/$defCmd -s $nodeOpt $hostname");
	#foreach my $line (@$output){
	#  if($line =~ m/=/){
	#	$line =~ s/\s//;#remove all whitespace from the line
	#	my ($attr, $value) = split "=", $line;
	#	$values{$attr}=$value;
	#  }
	#}
  }

my $code = GuiUtils->processHTML('definenode-src.html');
eval $code;
if (length($@)) { print "<P>$@</P>"; }

&footer('.', $::DEV?'Devices':'CSM Home');


