#!/usr/bin/perl
#
#    Cluster Systems Management Webmin Module
#    Copyright (C) 2002 by IBM Corp.
#
# Set physical location for nodes

use strict;
BEGIN { $::CSMDIR = '..'; }
use lib "$::CSMDIR";
require GuiUtils;
GuiUtils->init();
#%access=&get_module_acl;

GuiUtils->addHeader('rack', 0, exists($::in{setLocButton})?'Setting Location':'Set Location');

#my %values;
#my $changedAttrStr;
#todo: use lsrsrcdef-api to determine if PhysicalLocation attr exists
my $locAttr = 'UserComment';

if (exists($::in{setLocButton})) {          # setting location of nodes
  print "<H2>Diagnostic Commands and Output:</H2>\n";
  #my %orig = split(/:\|:/, $::in{origValues});

  if ($::in{algorithm} eq 'nodename' || $::in{algorithm} eq 'noderange') {
	# The noderange algorithm is similar to the nodename algorithm, except that
	# they tells us the nodes and we only do one rack.
	my $oneRack = $::in{algorithm} eq 'noderange' ? $::in{RackNum} : 0;
	my $nodes;
	if ($oneRack) { $nodes = NodeUtils->gatherNodeList([$::in{NodeRange}]); ($nodes) = NodeUtils->resolveAndUndup($nodes);}
	else { $nodes = NodeUtils->listNodeAttrs(undef, 'Hostname'); }
	@$nodes = sort @$nodes;

	# Calculate position and build chrsrc-api string
	my $chstr;
	my $nodeu = ($oneRack ? $::in{UPerNode2} : $::in{UPerNode}) || 1;
	my $perRack = $::in{NodesPerRack} || int(42/$nodeu);
	my ($u, $nodenum) = (1, 1);
	my $rack = $oneRack || 1;
	foreach my $n (@$nodes) {
	  my $value = "Rack=$rack U=$u";
	  #todo: handle cmd line too long
	  $chstr .= qq( -s IBM.ManagedNode::"Hostname=='$n'"::${locAttr}::'$value');

	  # increment positions
	  $nodenum++;
	  $u += $nodeu;
	  if ($nodenum > $perRack || $u > 42) {
		if ($oneRack) { last; }
		$rack++; $u = $nodenum = 1;
	  }
	}

	if (length($chstr))
	  {
		my $outref = GuiUtils->runcmd(qq(/usr/bin/chrsrc-api $chstr));
		#if (!$::RUNCMD_RC) { $changedAttrStr = 'Properties changed: ' . join(', ', keys %changedAttrs); }
		#else { $changedAttrStr = 'No nodes were changed.'; }
	  }
  }   # if nodename or noderange

  elsif ($::in{algorithm} eq 'bladechassis') {
	my $nodes;
	if (length($::in{ChassisRange})) {
	  $::NODEGROUP_CLASS = 'IBM.DeviceGroup';     # this global variable is used by the group expansion functions in NodeUtils
	  $nodes = NodeUtils->gatherNodeList([$::in{ChassisRange}]);
	  ($nodes) = NodeUtils->resolveAndUndup($nodes);
	}
	else {
	  $nodes = NodeUtils->listNodeAttrs(undef, 'Name',{ResourceClass => 'IBM.DeviceHwCtrl', WhereStr => "DeviceType IN ('MM','mm')"});
	}
	@$nodes = sort @$nodes;

	# Calculate position and build chrsrc-api string
	my $chstr;
	my $nodeu = 7;
	my $perRack = $::in{ChassisPerRack} || int(42/$nodeu);
	my ($u, $nodenum) = (1, 1);
	my $rack = $::in{BladeRackNum} || 1;
	foreach my $n (@$nodes) {
	  my $value = "Rack=$rack U=$u";
	  #todo: handle cmd line too long
	  $chstr .= qq( -s IBM.DeviceHwCtrl::"Name=='$n'"::${locAttr}::'$value');

	  # increment positions
	  $nodenum++;
	  $u += $nodeu;
	  if ($nodenum > $perRack || $u > 42) { $rack++; $u = $nodenum = 1; }
	}

	if (length($chstr))
	  {
		my $outref = GuiUtils->runcmd(qq(/usr/bin/chrsrc-api $chstr));
		#if (!$::RUNCMD_RC) { $changedAttrStr = 'Properties changed: ' . join(', ', keys %changedAttrs); }
		#else { $changedAttrStr = 'No nodes were changed.'; }
	  }
  }   # if bladechassis

  print "<P><BR></P>\n";
  delete($::in{setLocButton});
  my $params = &urlize(join('&', GuiUtils->hash2array(%::in)));
  &footer("rack.cgi?$params", 'Set Location');
}

else {     # We are 1st coming to this page, display it
  #my $cmd = qq(lsrsrc-api -D ':|:' -i -n -s IBM.Condition::"Name IN ('$condition')"::Name::NodeNames::ManagementScope::ResourceClass::SelectionString::EventExpression::RearmExpression::MonitorStatus::EventOccurred::Severity::ImmediateEvaluate);
  #my $outref = GuiUtils->runcmd('/opt/csm/bin/csmconfig');

  #%values = split(/:\|:/, $$outref[0]);      # this puts the pairs of attr name and value in the hash correctly
  #foreach my $l (@$outref)
  #  {
  #	my ($a, $v) = $l=~/^\s*(\S+)\s*=\s*(.*)$/;
  #	$values{$a} = $v;
  #  }

  my $algorithmNum;
  if (!length($::in{algorithm}) || $::in{algorithm} eq 'nodename') { $algorithmNum = 0; }
  elsif ($::in{algorithm} eq 'noderange') { $algorithmNum = 1; }
  elsif ($::in{algorithm} eq 'bladechassis') { $algorithmNum = 2; }

  my $code = GuiUtils->processHTML('rack-src.html');
  eval $code;
  if (length($@)) { print "<P>$@</P>"; }

  &footer('', 'CSM Home');
}

