#!/usr/bin/perl
#
#    Cluster Systems Management Webmin Module
#    Copyright (C) 2002 by IBM Corp.
#
# Display main page of CSM Web Interface

use strict;
use lib '$ENV{PWD}';
require GuiUtils;
GuiUtils->init();

# Build node list
#my $nodeErr;
my ($options, @nodes);

# Go to the page corresponding to the button
if ($::in{propButton}) { &redirect("properties.cgi?" . getParams()); }
elsif ($::in{defineButton}) { &redirect("definenode.cgi?" . getParams()); }
elsif ($::in{updateButton}) { &redirect("update.cgi?" . getParams()); }
elsif ($::in{runcmdButton})
  {
	my $p = getParams();
	if ($p =~ /nodes=./) { &redirect("dsh.cgi?$p"); }          # some nodes were selected
	else { &redirect('../shell'); }                                  # no nodes selected
  }
elsif ($::in{copyFilesButton}) { &redirect("dcp.cgi?" . getParams()); }
elsif ($::in{pingNodesButton}) { &redirect("dping.cgi?" . getParams()); }
elsif ($::in{createGroupButton}) { &redirect("nodegrp.cgi?" . getParams() . "&groupType=staticType"); }
elsif ($::in{hwctrlButton}) { &redirect("hwctrl/index.cgi?" . getParams()); }
elsif ($::in{rsaButton})
  {
	my $node = getFirstNode();
	if ($node) {
	  my $nodeAttrs = NodeUtils->listNodeAttrs([$node], 'HWControlPoint');
	  my $hcp = $$nodeAttrs[0];
	  if (length($hcp)) { &redirect("http://$hcp"); }
	  else { &redirect('jserror.cgi?error=' . &urlize("The HWControlPoint attribute is not defined for $node.")); }
	}
  }
elsif ($::in{perfmonButton}) { &redirect("perfmon/index.cgi?" . getParams()); }
elsif ($::in{webminButton}) { &redirect('http://' . getFirstNode() . ':10000'); }
elsif ($::in{softMaintButton}) { &redirect("softmaint/node.cgi?" . getParams()); }
elsif ($::in{diagButton}) { &redirect("diagnode/index.cgi?" . getParams()); }
elsif ($::in{installButton}) { &redirect("install/index.cgi?" . getParams()); }
elsif ($::in{removeButton}) { &redirect("remove.cgi?" . getParams()); }
# else they are coming to this page from elsewhere, so display it...


$::VERBOSE = $::GUI_VERBOSE;          # this will cause NodeUtils to show the cmds
$ENV{'CT_MANAGEMENT_SCOPE'}=1;
if (length($::config{CSM_NODE_PREFIX})) { $ENV{CSM_NODE_PREFIX} = $::config{CSM_NODE_PREFIX}; }

#%access=&get_module_acl;
## put in ACL checks here if needed

# Create title and header area
GuiUtils->addHeader('intro', 0);
#&header('Cluster Systems Management', 'images/csmlogo.gif', 'intro', 1, 1, undef,
#        '<A HREF=mailto:cluster\@us.ibm.com?Subject=CSM Web Interface Suggestion>Send suggestions to the authors</A>');

#print "config_directory=$::config_directory\n";
#if ($nodeErr) { GuiUtils->msg('E', $nodeErr); }

# Use the users cookie values as defaults if the corresponding value is not in the URL
my ($cookies) = &::un_urlize($ENV{HTTP_COOKIE}) =~ /mainpage=([^;]+)/;
my ($rackCookie, $nodegrpCookie, $nodeRangeCookie) = split(/&/, $cookies);
if (!exists($::in{rack}) && defined($rackCookie)) { $::in{rack} = $rackCookie; }
if (!exists($::in{group}) && defined($nodegrpCookie)) { $::in{group} = $nodegrpCookie; }
if (!exists($::in{nodeRange}) && defined($nodeRangeCookie)) { $::in{nodeRange} = $nodeRangeCookie; }

# Get list of nodes and attributes and store for the HTML
my $noderef;
my $opthashref;
if ($::in{nodeRange} =~ /\S/) { $noderef = NodeUtils->gatherNodeList([$::in{nodeRange}]); }
elsif (exists($::in{group}) && $::in{group} ne 'AllNodes')
  {
	$noderef = NodeUtils->gatherNodeList(undef, { NodeGrp => $::in{group} });
	$opthashref = { SkipResolve => 1, Res2Unres => {} };   # node groups are already resolved and unduped
  }
my $attrs = 'Hostname,HWType,InstallDistributionName,InstallDistributionVersion,Mode,Status,ConsolePortNum,HWControlPoint,HWControlNodeId,UserComment';
my $nodeAttrs;
if (defined($noderef) && !scalar(@$noderef)) { $nodeAttrs = []; }     # empty group
else { $nodeAttrs = NodeUtils->listNodeAttrs($noderef, $attrs, $opthashref); }

# Get list of node group names for the select menu
my $nodegrp = GuiUtils->runcmd(qq(/usr/bin/lsrsrc-api -i -D ':|:' -s IBM.NodeGroup::::Name));
@$nodegrp = sort @$nodegrp;

# Prepare the data structures for the html file
my $rackChecked = $::in{rack} ? 1 : 0;
my @frames;
if (!$rackChecked) {
  @$nodeAttrs = sort @$nodeAttrs;
  #foreach my $n (@$nodeAttrs) { print "<P>$n</P>\n"; }
}
else {   # form the frame arrays
  foreach my $na (@$nodeAttrs)
	{
	  my ($hostname, $type, $distro, $version, $mode, $status, $conport, $hcp, $nodeid, $comment) = split(/:\|:/, $na);

	  my ($slot, $loc, $size);
	  my $isBlade=0;
	  my $image = GuiUtils->getHWTypeImage($type);
	  if ($image eq 'blade.gif') {
		$isBlade = 1;
		$image = 'blade-front-';
		$size = 7;
		$loc = getMMLoc($hcp);
		if ($conport =~ /^\s*\d{1,2}\s*$/) { $slot = $conport; }
		elsif ($nodeid =~ /^\s*\d{1,2}\s*$/) { $slot = $nodeid; }       # we interpret a 2 digit nodeid as the slot #
		else { next; }       # do not know the position of the blade
	  }
	  else {   # non-blade
		if ($image eq '342.gif') { $image = 'x345-front-'; $size=2; }
		else { $image = 'x335-front-'; $size=1; }
		$loc = $comment;
	  }
	  #print "<p>hostname=$hostname, isBlade=$isBlade, loc=$loc, slot=$slot</p>\n";

	  my ($junk, $f) = $loc=~/(frame|rack)=(\d+)/i;
	  if (!defined($f)) { next; }
	  my ($u) = $loc=~/u=(\d+)/i;
	  if (!defined($u)) { next; }

	  my $alt = "$hostname: ";
	  if ($distro=~/redhat/i) { $alt .= "RHEL $version"; }
	  elsif ($distro=~/suse|sles/i) { $alt .= "SLES $version"; }
	  else { $alt .= "$distro $version";}
	  $alt .= ", U=$u";
	  if ($isBlade) { $alt .= " Slot=$slot"; }

	  if ($status == 1) { $image .= 'green.jpg'; }
	  elsif ($status == 127 && $mode eq 'PreManaged') { $image .= 'blue.jpg'; }
	  elsif ($status == 127) { $image .= 'yellow.jpg'; }
	  else { $image .= 'red.jpg'; }

	  if (!defined($frames[$f])) { $frames[$f] = []; }    # start an array for this frame
	  my $frame = $frames[$f];
	  my $findex = 43-$u-($size-1);
	  if ($isBlade) {
		if (!defined($$frame[$findex]) || !ref($$frame[$findex]->[1])) { $$frame[$findex] = [$hcp, [], undef, $size]; }
		my $chassis = $$frame[$findex]->[1];
		$$chassis[$slot] = [$hostname, $image, $alt];
	  }
	  else { $$frame[$findex] = [$hostname, $image, $alt, $size]; }
	}
}

# Get versions of csm.server and xcsm.web
my %rpmVersions = GuiUtils->getInstalledVersions(1, qw(csm.server xcsm.web));

# Process the perl code within the HTML
#print '<P>Lsnode output: ' . join(',',@out) . '</P>';
my $code = GuiUtils->processHTML('index-src.html');
#open WEBCODE, '>/tmp/index-src.html'; select WEBCODE;
eval $code;
if (length($@)) { print "<P>$@</P>"; }

#if ($nodeErr) { print qq(<script language="JavaScript">if (document.getElementById('displayederror')) {alert('yes');} else {alert('no');}</script><div ID=displayederror></div>); }

&footer("/", 'Webmin Cluster Index');


sub getParams
  {
	my @nodes;
	my @params;
	my $individual = 1;
	if (length($::in{Nodes})) { push @nodes, $::in{Nodes}; $individual=0; }
	elsif (length($::in{rackNodes})) { push @nodes, split(/,/,$::in{rackNodes}); $individual=0; }
	foreach my $k (keys %::in)
	  {
		if ($individual && $k =~ /^node\d+/) { push @nodes, $::in{$k}; }
		elsif (!grep(/^$k$/, qw(nodegrps nodeRange Nodes rackNodes)) && $k !~ /^node\d+/) { push @params, "$k=" . &urlize($::in{$k}); }
	  }
	if (scalar(@nodes)) {
	  @nodes = sort @nodes;
	  push @params, 'nodes=' . &urlize(join(',', @nodes));
	}
	return join('&', @params);
  }

sub getFirstNode
  {
	if (length($::in{Nodes})) {
	  my $nodelist = GuiUtils->processNodelist($::in{Nodes}, (Array=>1,NoVerbose=>1));
	  if (!$nodelist) { return undef; }
	  else { return $$nodelist[0]; }
	}
	elsif (length($::in{rackNodes})) { my ($n) = split(/,/,$::in{rackNodes}); return $n; }
	else { foreach my $k (keys %::in) { if ($k =~ /^node\d+/) { return $::in{$k}; } } }
	return undef;
  }

sub getMMLoc {
  my $mm = shift;
  if (!defined(%::MMs)) {
	my $nodes = NodeUtils->listNodeAttrs(undef, 'Name,UserComment',{ResourceClass => 'IBM.DeviceHwCtrl', WhereStr => "DeviceType IN ('MM','mm')"});
	foreach my $n (@$nodes) { my ($name, $comment) = split(/:\|:/, $n);   $::MMs{$name} = $comment; }
	if (!scalar(@$nodes)) { $::MMs{_} = 1; }     # so we do not try to get it again
  }
  return $::MMs{$mm};
}
