#!/usr/bin/perl
#
#    Cluster Systems Management Webmin Module
#    Copyright (C) 2002 by IBM Corp.
#
# Display and modify node groups

use strict;
use lib $ENV{PWD};
require GuiUtils;
GuiUtils->init();

$ENV{'CT_MANAGEMENT_SCOPE'}=1;
$::DEV = exists($::in{device}) && $::in{device};
my $grpCmd = $::DEV ? 'hwdevgrp' : 'nodegrp';
my $lsCmd = $::DEV ? 'lshwdev' : 'lsnode';
my $option = $::DEV ? '-d' : '-n';
my $memName = $::DEV ? 'Device' : 'Node';
my $attrMan = $::DEV ? 'deviceattributes' : 'nodeattributes';
my $deviceChecked = $::DEV ? 'true' : 'false';
$::NODEGROUP_CLASS = $::DEV ? 'IBM.DeviceGroup' : undef;     # this global variable is used by the group expansion functions in NodeUtils

my $isAction = exists($::in{dynTestButton});
GuiUtils->addHeader(undef, 2, $isAction?'Try Dynamic Nodegroup Select String':undef);

# Process the buttons
if (exists($::in{trySelectButton}))
  {
	print "<H3>Select String Query:  $::in{selectString}</H3>\n";
	GuiUtils->runcmd("$GuiUtils::csmroot/bin/$lsCmd -w " . NodeUtils->quote($::in{selectString}), 1);
	print "<P></P>\n";
	&footer("nodegrp.cgi?groupName=$::in{groupName}&groupType=$::in{groupType}&selectString=" . &urlize($::in{selectString}) . "&device=$::in{device}", "CSM $memName Groups");
	return;
  }
elsif (exists($::in{chgroupButton}))
  {
	my $options;
	if ($::in{groupType} eq 'staticType')
	  {
		if (exists($::in{generic})) { $options .= ' -g'; }
		$options .= " $option " . NodeUtils->quote($::in{memberList});
	  }
	else    # dynamic
	  {
		$options .= ' -w ' . NodeUtils->quote($::in{selectString});
	  }
	GuiUtils->runcmd("$GuiUtils::csmroot/bin/$grpCmd $options $::in{groupName}");
	if ($::RUNCMD_RC == 0)
	  {
		GuiUtils->msg('I', "$::in{groupName} succesfully changed/created.");
		$::in{group} = $::in{groupName};
	  }
  }
elsif (exists($::in{delgroupButton}))
  {
	GuiUtils->runcmd("$GuiUtils::csmroot/bin/$grpCmd -D $::in{groupName}");
	if ($::RUNCMD_RC == 0)
	  {
		GuiUtils->msg('I', "$::in{groupName} succesfully removed.");
	  }
  }

# Get list of node groups
my $nodegroups = GuiUtils->runcmd("$GuiUtils::csmroot/bin/$grpCmd");
@$nodegroups = sort @$nodegroups;

# Get the attributes for the selected node group
my ($groupName, $groupType, $memberList, $selectString, $genericChecked, $staticChecked, $dynamicChecked);
if (exists($::in{group}))
  {
	my $outref = GuiUtils->runcmd("$GuiUtils::csmroot/bin/$grpCmd -L $::in{group}");
	my ($contents, $genericStr);
	($groupName, $groupType, $genericStr, $contents) = $$outref[0] =~ /^\s*(\S+):\s*([^,]+),\s*([^,]+),\s*(.*)$/;
	#print "<p>groupName=$groupName, groupType=$groupType, genericStr=$genericStr, contents=$contents</p>\n";
	if ($groupType =~ /static/i)     # create a static group
	  {
		$memberList = $contents;
		$genericChecked = ($genericStr =~ /not/i) ? 'true' : 'false';
		$staticChecked = 'true';
		$dynamicChecked = 'false';
	  }
	else          # dynamic group
	  {
		$selectString = $contents;
		$genericChecked = 'false';
		$staticChecked = 'false';
		$dynamicChecked = 'true';
	  }
  }
else            # no group selected
  {
	# handle the case where we are returning from the trySelectButton or invoked from the action button on the main page
	if (exists($::in{groupName})) { $groupName = $::in{groupName}; }
	$dynamicChecked = $::in{groupType} eq 'dynamicType' ? 'true' : 'false';
	$staticChecked = $::in{groupType} eq 'staticType' ? 'true' : 'false';
	$genericChecked = 'false';
	#if ($::in{groupType} eq 'dynamicType') { $dynamicChecked = 'checked'; }
	#elsif ($::in{groupType} eq 'staticType') { $staticChecked = 'checked'; }
	if (exists($::in{selectString})) { $selectString = $::in{selectString}; }
	if (exists($::in{memberList})) { $memberList = $::in{memberList}; }
	elsif (exists($::in{nodes})) {
	  $memberList = GuiUtils->processNodelist($::in{nodes}, (NodeGroup=>1));
	  $memberList =~ s/^-n\s*//;        # strip off the preceding -n
	}
  }
	
my $code = GuiUtils->processHTML('nodegrp-src.html');
eval $code;
if (length($@)) { print "<P>$@</P>"; }

&footer('', 'CSM Home');


