#!/usr/bin/perl
#
#    Cluster Systems Management Webmin Module
#    Copyright (C) 2002 by IBM Corp.
#
# Download and update software on the cluster

use strict;
BEGIN { $::CSMDIR = '..'; }
use File::Path;      # provides mkpath()
use lib "$::CSMDIR";
use GuiUtils;
#use lib "$GuiUtils::csmroot/pm";
#use InstallUtils;
GuiUtils->init();

# Go to the page corresponding to the button
#if ($::in{propButton}) { &redirect("cond-props.cgi?" . getParams()); }
my $isAction = keys %::in;

GuiUtils->addHeader(undef, 5, $isAction?'Run Soft Maint Cmds':undef, $isAction);

if ($isAction) { print "<H2>Software Maintenance Commands and Output:</H2>\n"; }

# Set some global variables for versions and directories
my ($distro, $version) = split(/\s/, NodeUtils->distribution());
if (exists($::in{reqrpmsButton}) && $::in{distroVersionTextfield}=~/\S/)
  {
	($distro, $version) = split(/\s/, $::in{distroVersionTextfield});
  }
my $tmpdir = '/tmp/csm';
my $tmpreqsdir = "$tmpdir/reqs";
my $tmpxcsmdir = "$tmpdir/xcsm";
#my $tmpdistrodir = "$tmpdir/$distro/$version";
#my $rpmdir = "/csminstall/Linux/$distro/$version/i386/$distro/RPMS";
#my $updatedir = "/csminstall/Linux/$distro/$version/i386/updates";
my $autodownload = '/usr/sbin/autoget --noinstall --noupdate --nodldaddinstall --noresolve --nopurge --nocleanup';
my $autoinstall = '/usr/sbin/autoins';


$::GUI_VERBOSE = 1;      # always want to show them the cmds we are running on this page
if (exists($::in{checkReqsButton}))     # check that we have all the software we need
  {
	# Check that autoupdate, perl-DB_File, xcsm, gnuplot, perl-DBI are installed on ms.
	# If not, try to install them.
	#todo:  what about perl-libnet?
	my @rpms = qw(autoupdate perl-DB_File xcsm gnuplot perl-DBI);
	my @optnotfound;
	my %rpmVersions = GuiUtils->getInstalledVersions(0, @rpms);
	foreach my $r (@rpms)
	  {
		if (exists($rpmVersions{$r})) { next; }       # already installed
		my $file = GuiUtils->findInPath("${r}-*.rpm", $::config{RPMPath});
		if (defined($file)) { GuiUtils->runcmd("/bin/rpm -i $file", 1); }
		else { push @optnotfound, $r; }
	  }
	if (scalar(@optnotfound)) { GuiUtils->msg('I', 'Optional RPMs were not installed on the management server and could not be found in the RPM path: ' . join(', ',@optnotfound)); }
	else { GuiUtils->msg('I', 'All optional RPMs are installed on the management server.'); }
  }
elsif (exists($::in{autoupdateButton}))     # download autoupdate
  {
	#todo:  other mirrors for autoupdate can be found at http://www.mat.univie.ac.at/~gerald/ftp/autoupdate/install.html
	my $host = 'ftp.mat.univie.ac.at';
	#my $host = '131.130.14.152';
	my $hostdir = '/pub/teschl/autoupdate';
	my $spec = 'autoupdate.spec';
	#my $file = 'autoupdate-4.7.2-1.noarch.rpm';
	mkpath($tmpreqsdir);
	my $errStr;

	# 1st download the spec file so we know the current version of autoupdate
	$::progress_callback_url = $spec;
	ftp_download($host, "$hostdir/$spec", "$tmpreqsdir/$spec", \$errStr, \&progress_callback);
	if (length($errStr)) { print "<P>$errStr</P>\n"; }
	else
	  {
		my $outref = GuiUtils->runcmd(qq(/bin/grep -E "^Version:|^.define vendor_release" $tmpreqsdir/$spec), 0, { NoVerbose => 1 });
		my ($release) = $$outref[0] =~ /^.define vendor_release\s*(\S+)\s*$/;
		my ($version) = $$outref[1] =~ /^Version:\s*(\S+)\s*$/;
		$version .= "-$release";
		print "<p>Latest autoupdate version is $version</p>\n";
		# now download the autoupdate rpm
		my $file = "autoupdate-$version.noarch.rpm";
		$::progress_callback_url = $file;
		#print "<P>Downloading ftp://$host$hostdir/$file ...</P>\n";
		ftp_download($host, "$hostdir/$file", "$tmpreqsdir/$file", \$errStr, \&progress_callback);
		if (length($errStr)) { print "<P>$errStr</P>\n"; }
		else
		  {
			GuiUtils->runcmd('rpm --import /usr/share/rhn/RPM-GPG-KEY');
			GuiUtils->runcmd("/bin/rpm -U $tmpreqsdir/$file");
		  }
	  }

	#todo: install perl-DB_File
  }
elsif (exists($::in{installmsButton})) { installms(); }     # install MS
elsif (exists($::in{installectButton})) { installect(); }     # install ECT
elsif (exists($::in{updatexCSMButton})) { updatexcsm(); }     # download install the xCSM rpm
elsif (exists($::in{updateCSMWebButton}))        # update this csm web interface
  {
	#todo: get this to work for the external site when it is at the techsupport site
	mkpath($tmpxcsmdir);
	my $externalSite = 'techsupport.services.ibm.com';
	my $externalURL = "ftp://$externalSite/eserver/pseries/cluster/csm/fixes";
	my $internalSite = 'rs6000.pok.ibm.com';
	my $internalURL = "http://$internalSite/afs/apd.pok.ibm.com/project/csm/install-images/xcsm";

	GuiUtils->msg('I', "Searching for the xcsm.web download site...");
	my $out = GuiUtils->runcmd(qq(ping -w 4 -c 1 $internalSite), 2, {NoVerbose=>1});
	if ($::RUNCMD_RC == 0) {        # they have access to internal IBM
	  GuiUtils->msg('I', "Downloading xcsm.web from the IBM internal site ($internalSite).");
	  GuiUtils->runcmd("$autoinstall --updatedir $tmpxcsmdir --url $internalURL xcsm.web", 1, {NoVerbose=>1});
	}
	else {        # download from external site
	  GuiUtils->msg('I', "Downloading xcsm.web from the IBM CSM support site ($externalSite).");
	  GuiUtils->runcmd("$autoinstall --updatedir $tmpxcsmdir --url $externalURL xcsm.web", 1, {NoVerbose=>1});
	}
  }
elsif (exists($::in{webminButton}))     # download webmin
  {
	my $ftpsite = 'http://unc.dl.sourceforge.net/sourceforge/webadmin';
	mkpath($tmpxcsmdir);
	#GuiUtils->runcmd("$autodownload --updatedir $tmpxcsmdir --rpmdir $tmpxcsmdir --url $ftpsite $rpmlist", 1, {NoVerbose=>1});
	GuiUtils->runcmd("$autoinstall --updatedir $tmpxcsmdir --url $ftpsite webmin", 1, {NoVerbose=>1});
  }

if ($isAction)
  {
	print "<P><BR></P>\n";
	&footer('.', 'Software Maint');
  }
else
  {
	my %rpmVersions = GuiUtils->getInstalledVersions(1, qw(csm.server autoupdate xcsm.web webmin xcsm));
	my $code = GuiUtils->processHTML('index-src.html');
	eval $code;
	if (length($@)) { print "<P>$@</P>"; }
	&footer('', 'CSM Home');
  }



sub installms
  {
	# Explode tarball
	GuiUtils->msg('I', "Looking in $tmpdir for install images...");
	chdir($tmpdir);
	my $tarballpattern = 'csm-linux-*-GA.*tar.gz';
	my $tarball = GuiUtils->getLatestFile($tarballpattern);
	if (!$tarball) { print "<p>File $tarballpattern not found.  Please download it to /tmp/csm first.</p>\n";  return; }
	my ($version) = $tarball =~ /^csm-linux-(.+?)-/;
	if (!GuiUtils->getLatestFile("csm.server-$version-*.i386.rpm"))
	  {
		# Tarball has not been exploded yet
		GuiUtils->runcmd("/bin/tar -zxvf $tarball", 1, {NoVerbose=>1});
	  }
	else { print "<p>The CSM RPMs have already be extracted from $tarball.</p>\n"; }
	
	# Install csm.core
	my $csmcore = GuiUtils->getLatestFile("csm.core-*.i386.rpm");
	if (!GuiUtils->isInstalled($csmcore))
	  { GuiUtils->runcmd("/bin/rpm -U $csmcore", 1, {NoVerbose=>1}); }
	else { print "<p>$csmcore is already installed.</p>\n"; }
	
	# Run installms
	# The 1 piped into installms is in case another tftp program is installed, we tell it
	# to remove that and install tftp-hpa.
	GuiUtils->runcmd(qq(/bin/echo -e "1\n\n" | $GuiUtils::csmroot/bin/installms -p $::config{RPMPath}), 1);
  }


sub installect
  {
	#chdir($tmpxcsmdir);
	
	# Install xcsm
	installFromPath("xcsm-*.noarch.rpm");
	
	# Install webmin
	#installFromPath("webmin-*.noarch.rpm");
	
	# Install xcsm.web
	installFromPath("xcsm.web-*.noarch.rpm");
  }


sub updatexcsm
  {
	#todo: get this to work for the external site when it is at the techsupport site
	mkpath($tmpxcsmdir);
	
	my $internalSite = 'http://rs6000.pok.ibm.com/afs/apd.pok.ibm.com/project/csm/install-images/xcsm';
	GuiUtils->runcmd("$autoinstall --updatedir $tmpxcsmdir --url $internalSite xcsm", 1, {NoVerbose=>1});
  }


sub installFromPath
  {
	my $rpm = shift @_;
	my $file = GuiUtils->findInPath($rpm, $::config{RPMPath});
	if (defined($file))
	  {
		if (!GuiUtils->isInstalled($file)) { GuiUtils->runcmd("/bin/rpm -U $file", 1, {NoVerbose=>1}); }
		else { GuiUtils->msg('I', "$file is already installed."); }
	  }
	else { GuiUtils->msg('E', "Can not find $rpm RPM in $::config{RPMPath}"); }
  }
