#!/usr/bin/perl
#
#    Cluster Systems Management Webmin Module
#    Copyright (C) 2002 by IBM Corp.
#
# Download and update software for a node type

use strict;
BEGIN { $::CSMDIR = '..'; }
use File::Path;      # provides mkpath()
use lib "$::CSMDIR";
use GuiUtils;
use lib "$GuiUtils::csmroot/pm";
use InstallUtils;
GuiUtils->init();

# Go to the page corresponding to the button
#if ($::in{propButton}) { &redirect("cond-props.cgi?" . getParams()); }
my $isAction = 0;
foreach my $k (keys %::in) { if ($k=~/Button$/ && $k ne 'softMaintButton') { $isAction = 1; last; } }

GuiUtils->addHeader(undef, 0, $isAction?'Run Soft Maint Cmds':'Maintain Nodes');

if ($isAction) { print "<H2>Software Maintenance Commands and Output:</H2>\n"; }

# Set some global variables for versions and directories
my ($os, $distro, $version, $arch, $service);
my $nodelist;
if (exists($::in{InstallDistributionName}))       # these values are being passed from the main soft maint page
  {
	($os, $distro, $version, $arch, $service) = ('Linux',$::in{InstallDistributionName},$::in{InstallDistributionVersion},$::in{InstallPkgArchitecture},$::in{InstallServiceLevel});
  }
else         # get the values from the node
  {
	$::VERBOSE = $::GUI_VERBOSE;          # this will cause NodeUtils to show the cmds
	#my @node_array = split(/,/, $::in{nodes});
	$nodelist = GuiUtils->processNodelist($::in{nodes}, (listNodeAttrs=>1));
	my $attr_ref = NodeUtils->listNodeAttrs($nodelist, 'InstallOSName,InstallDistributionName,InstallDistributionVersion,InstallPkgArchitecture,InstallServiceLevel');
	$::VERBOSE = 0;
	($os, $distro, $version, $arch, $service) = split(/:\|:/, $$attr_ref[0]);
  }
#my ($os, $distro, $version, $arch, $service) = ($attr{InstallOSName},$attr{InstallDistributionName},$attr{InstallDistributionVersion},$attr{InstallPkgArchitecture},$attr{InstallServiceLevel});
my $servdir = length($service) ? "/$distro$version-$service" : '';
my $tmpdir = '/tmp/csm';
my $tmpreqsdir = "$tmpdir/reqs";
#my $tmpdistrodir = "$tmpdir/$distro/$version";
my $rpmdir = "/csminstall/$os/$distro/$version/$arch$servdir/$distro/RPMS";
#my $updatedir = "/csminstall/$os/$distro/$version/$arch$servdir/updates";
my $updatedir = "/csminstall/$os/$distro/$version/$arch/updates";
#my $installdir = "/csminstall/$os/$distro/$version/$arch$servdir/install";
my $installdir = "/csminstall/$os/$distro/$version/$arch/install";
my $autodownload = '/usr/sbin/autoget --noinstall --noupdate --nodldaddinstall --noresolve --nopurge --nocleanup';
my $autoinstall = '/usr/sbin/autoins';


$::GUI_VERBOSE = 1;      # always want to show them the cmds we are running on this page
if (exists($::in{checkReqsButton}))     # check that we have all the software we need
  {
	# Check base CSM rpms and prereqs
	# I would like to use the copycsmpkgs command to check, but it copies stuff every time and will prompt for CD if something is missing
	$::ATTRS{'InstallOSName'} = $os;
	$::ATTRS{'InstallDistributionName'} = $distro;
	$::ATTRS{'InstallDistributionVersion'} = $version;
	$::ATTRS{'InstallPkgArchitecture'} = $arch;
	$::ATTRS{'InstallServiceLevel'} = $service;
	#$::ATTRS{'InstallCSMVersion'} = '';
	ArchiveUtils->getPackageList();
	#GuiUtils->msg('I', 'Done getting list of CSM prerequisites.');
	my @opensrc;
	foreach my $r (@::opensrc_rpm_prereqs)
	  {
		if ($r !~ /(ITDAgent|DirAgent|RAIDLxAg|MPAAgent)/) { push @opensrc, $r; }
	  }
	GuiUtils->msg('O', '<P>CSM required and optional softare:<UL><LI>CSM pkgs: ' . join(', ',@::csm_packages) . '<LI>RSCT pkgs: ' . join(', ',@::rsct_packages) . '<LI>Distro reqs: ' . join(', ',@::os_prereq) . '<LI>Open src reqs: ' . join(', ',@opensrc) . '</UL>');

	# Now try to find these in the rpm path
	GuiUtils->msg('O', "<P>Looking for CSM required and optional softare in $::config{RPMPath}...");
	my (@rpms, @reqnotfound, @optnotfound);
	push @rpms, @::csm_packages, @::rsct_packages, @::os_prereq, @opensrc;
	foreach my $r (@rpms)       # each required rpm
	  {
		my $f = "${r}*.rpm";
		$f =~ s/\*\*/\*/;
		if (!GuiUtils->findInPath($f, $::config{RPMPath}))
		  {
			if ($r =~ /(autoupdate|perl-DB_File|perl-libnet|perl-DBI|fping|perl-XML-Parser|perl-XML-Simple)/) { push @optnotfound, $r; }
			else { push @reqnotfound, $r; }
		  }
	  }
	if (!scalar(@reqnotfound) && !scalar(@optnotfound))
	  {
		GuiUtils->msg('O', '<BR>All CSM software found.');
		if ($::in{runCopycsmpkgs})
		  {
			#todo: maybe should pipe f (for force) to it in case it prompts for a CD
			GuiUtils->runcmd("/opt/csm/bin/copycsmpkgs -p $::config{RPMPath} InstallDistributionName=$distro InstallDistributionVersion=$version InstallPkgArchitecture=$arch InstallServiceLevel=$service", 1);
		  }
	  }
	else
	  {
		if (scalar(@reqnotfound)) { GuiUtils->msg('E', 'Required RPMs not found in the RPM path: ' . join(', ',@reqnotfound)); }
		if (scalar(@optnotfound)) { GuiUtils->msg('I', 'Optional RPMs not found in the RPM path: ' . join(', ',@optnotfound)); }
	  }
  }
elsif (exists($::in{reqrpmsButton}))     # download RPM prereqs
  {
	mkpath($rpmdir);
	my ($ddir, $vdir, $rpmlist);
	if ($::in{reqrpmsTextfield}=~/\S/) { $rpmlist = $::in{reqrpmsTextfield};  $rpmlist =~ s/,/ /g; }
	else
	  {
		# I would like to use the copycsmpkgs command to check, but it copies stuff every time and will prompt for CD if something is missing
		$::ATTRS{'InstallOSName'} = $os;
		$::ATTRS{'InstallDistributionName'} = $distro;
		$::ATTRS{'InstallDistributionVersion'} = $version;
		$::ATTRS{'InstallPkgArchitecture'} = $arch;
		$::ATTRS{'InstallServiceLevel'} = $service;
		#$::ATTRS{'InstallCSMVersion'} = '';
		ArchiveUtils->getPackageList();
		$rpmlist = join(' ', @::os_prereq);
		GuiUtils->msg('I', "CSM required and optional distro RPMs: $rpmlist");
	  }
	if ($distro eq 'RedHat')
	  {
		$vdir = $version;
		$ddir = $distro;
		#if (defined($rpmlist)) {}   # the user specified the rpms to download
		#elsif ($version eq '7.2' || $version eq '7.3') { $rpmlist = 'freetype pdksh perl rdist nfs-utils dhcp expect tcl tk XFree86-libs perl-libnet'; }
		#elsif ($version eq '8.0') { $rpmlist = 'glibc glibc-common syslinux freetype pdksh perl rdist nfs-utils dhcp expect tcl tk XFree86-libs'; }
		#elsif ($version eq '9') { $rpmlist = 'glibc glibc-common compat-libstdc++ syslinux freetype pdksh perl rdist nfs-utils dhcp expect tcl tk XFree86-libs'; }
		#elsif ($version eq 'AS2.1') { $rpmlist = 'freetype pdksh perl rdist nfs-utils dhcp expect tcl tk XFree86-libs perl-libnet'; }
		#else { $rpmlist = 'freetype pdksh perl rdist nfs-utils dhcp expect tcl tk XFree86-libs perl-libnet'; }
	  }
	#elsif ($distro =~ /^RedHatEL/)
	#  {
	#	my ($edition) = $distro =~ /^RedHatEL-(\S+)/;
	#	if ($version eq '3')
	#	  {
	#		$vdir = "enterprise/$version$edition";
	#		$ddir = $distro;
	#		if (!defined($rpmlist)) { $rpmlist = ''; }
	#	  }
	#  }
	else { GuiUtils->msg('E', "Do not support downloads for $distro $version."); }

	if (defined($vdir) && defined($ddir) && defined($rpmlist))
	  {
		my $ftpsite = "http://ftp.redhat.com/pub/redhat/linux/$vdir/en/os/i386/$ddir/RPMS";
		my $updatessite = "http://updates.redhat.com/$vdir/en/os/i386";
		GuiUtils->runcmd("$autodownload --updatedir $rpmdir --rpmdir $rpmdir --url $updatessite $rpmlist", 1, {NoVerbose=>1});
		GuiUtils->runcmd("$autodownload --updatedir $rpmdir --rpmdir $rpmdir --url $ftpsite $rpmlist", 1, {NoVerbose=>1});
	  }
  }
elsif (exists($::in{webminButton}))     # copy webmin
  {
	my $file = GuiUtils->findInPath("webmin-*.noarch.rpm", $::config{RPMPath});
	if (defined($file))
	  {
		mkpath($installdir);
		GuiUtils->runcmd("/bin/cp $file $installdir", 1);
	  }
	else { GuiUtils->msg('E', 'Could not find the webmin RPM in the RPM path.'); }
  }
elsif (exists($::in{appButton}))     # copy apps
  {
	my @rpms = split(/[\s,]+/, $::in{appTextfield});
	GuiUtils->msg('I', "Looking for specified RPMs in $::config{RPMPath}...");
	my (@found, @notfound);
	foreach my $r (@rpms)
	  {
		my $f = "${r}-*.rpm";
		#$f =~ s/\*\*/\*/;
		my $file = GuiUtils->findInPath($f, $::config{RPMPath});
		if (defined($file)) { push @found, $file; }
		else { push @notfound, $r; }
	  }
	if (scalar(@found))
	  {
		mkpath($installdir);
		GuiUtils->runcmd('/bin/cp ' . join(' ',@found) . " $installdir", 1);
	  }
	if (scalar(@notfound)) { GuiUtils->msg('E', 'RPMs not found in the RPM path: ' . join(', ',@notfound)); }
  }
elsif (exists($::in{redhatUpdatesButton}))     # download RedHat updates
  {
	my $ftpsite = "http://ftp.redhat.com/pub/redhat/linux/updates/$version/en/os/i386";
	GuiUtils->runcmd("/usr/sbin/autodld --noinstall --noupdate --nodldaddinstall --updatedir $updatedir --rpmdir $rpmdir --url $ftpsite", 1, {NoVerbose=>1});
  }

if ($isAction)
  {
	print "<P><BR></P>\n";
	&footer("node.cgi?nodes=" . &urlize($::in{nodes}), 'Software Maint');
  }
else
  {
	#my %rpmVersions = GuiUtils->getInstalledVersions(1, qw(csm.server autoupdate xcsm.web webmin xcsm));
	my $code = GuiUtils->processHTML('node-src.html');
	eval $code;
	if (length($@)) { print "<P>$@</P>"; }
	&footer('', 'CSM Home');
  }

